@@ -1,5 +1,5 @@
-/*  Rational - Rational number class with overflow detection.
-    Copyright (C) 2012, 2013, 2014 Antonio Diaz Diaz.
+/*  Rational - Rational number class with overflow detection
+    Copyright (C) 2012-2015 Antonio Diaz Diaz.
 
     This program is free software: you have unlimited permission to
     copy, distribute and modify it.
@@ -31,7 +31,7 @@ namespace {
 
 const char * const Program_name = "Rational";
 const char * const program_name = "rational";
-const char * const program_year = "2014";
+const char * const program_year = "2015";
 const char * invocation_name = 0;
 
 
@@ -46,6 +46,7 @@ void show_help()
                "  -V, --version                output version information and exit\n"
                "  -p, --pi                     calculate approximations of pi\n"
                "  -q, --quiet                  quiet operation\n"
+               "  -s, --sqrt=<n>               calculate approximations of sqrt(n)\n"
                "  -t, --test=<level>           run tests at given level (1-9)\n"
                "  -v, --verbose                verbose operation\n"
                "\nReport bugs to arg-parser-bug@nongnu.org\n"
@@ -85,6 +86,73 @@ void internal_error( const char * const msg )
   }
 
 
+void approximate_pi()
+  {
+  const std::string target( "3.14159265358979323846264338327950288419" );
+  const int tlen = target.size();
+  int max_len = 1;
+  int n = 3, d = 1, digit = 0;
+
+  std::printf( "target == %s\n", target.c_str() );
+  std::printf( "correct\ndecimals                  value\n" );
+  while( n < INT_MAX )
+    {
+    if( n / d != 3 ) { if( n / d < 3 ) ++n; else ++d; continue; }
+    long long rest = n % d;
+    int i = 2;
+    for( ; i < tlen; ++i )
+      {
+      rest *= 10; digit = '0' + ( rest / d ); rest %= d;
+      if( digit != target[i] ) break;
+      }
+    if( i > max_len )
+      {
+      const Rational r( n, d );
+      max_len = i;
+      std::printf( "%2d      %s (%s)\n", i - 2,
+                   r.to_decimal( 1, -(tlen - 2) ).c_str(),
+                   r.to_fraction().c_str() );
+      std::fflush( stdout );
+      if( max_len >= tlen ) break;
+      }
+    if( digit < target[i] ) ++n;
+    else ++d;
+    }
+  }
+
+
+void approximate_sqrt( const int itarget )
+  {
+  const Rational target( itarget );
+  Rational best_value( 1 ), best_error( itarget - 1 );
+  const int prec = 10;
+  int n = 1, d = 1;
+
+  std::printf( "target == sqrt( %d )\n", itarget );
+  std::printf( "        error               value^2          value\n" );
+  while( n < INT_MAX )
+    {
+    const Rational value( n, d );
+    Rational value2( value ); value2 *= value2;
+    Rational error( value2 ); error -= target;
+    if( value2.error() || error.error() ) break;
+    if( error.abs() < best_error && error.abs() < 1 )
+      {
+      best_value = value; best_error = error.abs();
+      std::printf( "%-21s    %-12s    %-12s (%s)\n",
+                   error.to_decimal( 2, -( prec + 8 ) ).c_str(),
+                   value2.to_decimal( 1, -prec ).c_str(),
+                   value.to_decimal( 1, -prec ).c_str(),
+                   value.to_fraction().c_str() );
+      std::fflush( stdout );
+      if( best_error == 0 ) break;
+      }
+    if( error < 0 ) ++n;
+    else ++d;
+    }
+  }
+
+
 int run_tests( const int level, const bool verbose )
   {
   int retval = 0;
@@ -424,6 +492,7 @@ int run_tests( const int level, const bool verbose )
 int main( const int argc, const char * const argv[] )
   {
   int level = 0;
+  int sqrt_target = 0;
   bool pi_flag = false;
   bool verbose = false;
   invocation_name = argv[0];
@@ -433,6 +502,7 @@ int main( const int argc, const char * const argv[] )
     { 'h', "help",    Arg_parser::no  },
     { 'p', "pi",      Arg_parser::no  },
     { 'q', "quiet",   Arg_parser::no  },
+    { 's', "sqrt",    Arg_parser::yes },
     { 't', "test",    Arg_parser::yes },
     { 'v', "verbose", Arg_parser::no  },
     { 'V', "version", Arg_parser::no  },
@@ -453,6 +523,10 @@ int main( const int argc, const char * const argv[] )
       case 'h': show_help(); return 0;
       case 'p': pi_flag = true; break;
       case 'q': verbose = false; break;
+      case 's': sqrt_target = strtoul( arg, 0, 0 );
+                if( sqrt_target < 2 )
+                  { show_error( "Bad sqrt target", 0, true ); return 1; }
+                break;
       case 't': if( std::isdigit( *arg ) && *arg != '0' && !arg[1] )
                   level = *arg - '0';
                 else { show_error( "Bad test level", 0, true ); return 1; }
@@ -478,39 +552,7 @@ int main( const int argc, const char * const argv[] )
         retval = 1; }
     }
 
-  if( pi_flag )
-    {
-    const std::string target( "3.14159265358979323846264338327950288419" );
-    const int tlen = target.size();
-    int max_len = 1;
-    int n = 3, d = 1, digit = 0;
-
-    std::printf( "target == %s\n", target.c_str() );
-    std::printf( "correct\ndecimals                  value\n" );
-    while( n < INT_MAX )
-      {
-      if( n / d != 3 ) { if( n / d < 3 ) ++n; else ++d; continue; }
-      long long rest = n % d;
-      int i = 2;
-      for( ; i < tlen; ++i )
-        {
-        rest *= 10; digit = '0' + ( rest / d ); rest %= d;
-        if( digit != target[i] ) break;
-        }
-      if( i > max_len )
-        {
-        const Rational r( n, d );
-        max_len = i;
-        std::printf( "%2d      %s (%s)\n", i - 2,
-                     r.to_decimal( 1, -(tlen - 2) ).c_str(),
-                     r.to_fraction().c_str() );
-        std::fflush( stdout );
-        if( max_len >= tlen ) break;
-        }
-      if( digit < target[i] ) ++n;
-      else ++d;
-      }
-    }
-
+  if( pi_flag ) approximate_pi();
+  if( sqrt_target > 0 ) approximate_sqrt( sqrt_target );
   return retval;
   }
